// Standardausgabe verwenden
#include <iostream>
using namespace std;

#include <memory.h> // NULL


// Klasse, die speichert, wieviele Instanzen
// von ihr existieren
class CCount
{
  public:

    // Konstruktor: Anzahl um eins erhhen
    CCount()
    { m_instances++; }

    // Destruktor: Anzahl um eins verringern
    virtual ~CCount()
    {
        m_instances--;
        if (m_instances == 0)
            cout << "CCount sauber aufgeraeumt" << endl;
    }

    // Gebe Anzahl der Instanzen zurck
    static unsigned int instances()
    { return m_instances; }

  protected:

    // Anzahl erzeugter Objekte dieser Klasse
    static unsigned int m_instances;
};


// Statische Elemente ein Mal initialisieren
unsigned int CCount::m_instances = 0;


// Klasse, von der es hchstens eine Instanz gibt
class CSingleton
{
  public:

      // Einzige Instanz des Objekts zurckgeben
      static CSingleton* getSingleton()
      {
          // Objekt erzeugen, falls nicht existent
          if (m_refCount == 0)
              m_singleton = new CSingleton();

          m_refCount++;
          cout << "es gibt " << m_refCount << " Referenz(en)" << endl;

          return m_singleton; 
      }

      // Erzeugte Instanz des Objekts freigeben
      void releaseSingleton()
      {
          if (m_refCount > 0)
              m_refCount--;

          cout << "es gibt " << m_refCount << " Referenz(en)" << endl;

          // Objekt lschen, wenn nicht mehr verwendet
          if (m_refCount == 0)
              delete m_singleton;
      }

      void setValue(int newValue)
      { m_value = newValue; }

      int getValue()
      { return m_value; }

  protected:

    CSingleton()
    { cout << "Singleton Konstruktor" << endl; m_value = DEFAULT_VALUE; }

    virtual ~CSingleton()
    { cout << "Singleton Destruktor" << endl; }

    static unsigned int m_refCount;     // Anzahl der Referenzen auf das Objekt
    static CSingleton*  m_singleton;    // Einzige Instanz dieses Objekts

    int                 m_value;        // Vernderbarer Wert des Singleton Objekts
    static const int    DEFAULT_VALUE;  // Standardwert fr m_value
};

// Statische Elemente ein Mal initialisieren
unsigned int CSingleton::m_refCount    = 0;
CSingleton*  CSingleton::m_singleton   = NULL;
const int    CSingleton::DEFAULT_VALUE = 1;


int main()
{
    // Noch keine Instanzen erzeugt
    cout << CCount::instances() << endl;

    // Instanz erzeugen
    CCount c1;
    cout << CCount::instances() << endl;

    {
        // Mehr Instanzen erzeugen
        CCount c2[5];
        cout << CCount::instances() << endl;
    }

    // Gltigkeitsbereich des Arrays wurde verlassen
    cout << CCount::instances() << endl;

    CSingleton* s1 = CSingleton::getSingleton();
    CSingleton* s2 = CSingleton::getSingleton();

    cout << s2->getValue() << endl;
    s1->setValue(3);
    cout << s2->getValue() << endl;

    s2->releaseSingleton();
    s1->releaseSingleton();

    CSingleton* s3 = CSingleton::getSingleton();
    cout << s3->getValue() << endl;
    s3->releaseSingleton();

    // Gltigkeitsbereich der logalen Variablen wird verlassen
    return 0;

} // main()
